/*

  Broadcom B43legacy wireless driver

  debugfs driver debugging code

  Copyright (c) 2005 Michael Buesch <mb@bu3sch.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
  Boston, MA 02110-1301, USA.

*/



#include <linux/fs.h>
#include <linux/debugfs.h>
#include <linux/slab.h>
#include <linux/netdevice.h>
#include <linux/pci.h>
#include <linux/mutex.h>

#include "b43legacy.h"
#include "main.h"
#include "debugfs.h"
#include "dma.h"
#include "pio.h"
#include "xmit.h"

#define REALLY_BIG_BUFFER_SIZE	(1024 * 256)

static struct b43legacy_debugfs fs;
static char big_buffer[REALLY_BIG_BUFFER_SIZE];
static DEFINE_MUTEX(big_buffer_mutex);


static ssize_t write_file_dummy(struct file *file, const char __user *buf,
				size_t count, loff_t *ppos)
{
	return count;
}

static int open_file_generic(struct inode *inode, struct file *file)
{
	file->private_data = inode->i_private;
	return 0;
}

#define fappend(fmt, x...)	pos += snprintf(buf + pos, len - pos, fmt , ##x)

static ssize_t drvinfo_read_file(struct file *file, char __user *userbuf,
				 size_t count, loff_t *ppos)
{
	const size_t len = ARRAY_SIZE(big_buffer);
	char *buf = big_buffer;
	size_t pos = 0;
	ssize_t res;

	mutex_lock(&big_buffer_mutex);
	/* This is where the information is written to the "driver" file */
	fappend(KBUILD_MODNAME " driver\n");
	fappend("Compiled at: %s %s\n", __DATE__, __TIME__);
	res = simple_read_from_buffer(userbuf, count, ppos, buf, pos);
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t tsf_read_file(struct file *file, char __user *userbuf,
			     size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	const size_t len = ARRAY_SIZE(big_buffer);
	char *buf = big_buffer;
	size_t pos = 0;
	ssize_t res;
	unsigned long flags;
	u64 tsf;

	mutex_lock(&big_buffer_mutex);
	mutex_lock(&dev->wl->mutex);
	spin_lock_irqsave(&dev->wl->irq_lock, flags);
	if (b43legacy_status(dev) < B43legacy_STAT_STARTED) {
		fappend("Board not initialized.\n");
		goto out;
	}
	b43legacy_tsf_read(dev, &tsf);
	fappend("0x%08x%08x\n",
		(unsigned int)((tsf & 0xFFFFFFFF00000000ULL) >> 32),
		(unsigned int)(tsf & 0xFFFFFFFFULL));

out:
	spin_unlock_irqrestore(&dev->wl->irq_lock, flags);
	mutex_unlock(&dev->wl->mutex);
	res = simple_read_from_buffer(userbuf, count, ppos, buf, pos);
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t tsf_write_file(struct file *file, const char __user *user_buf,
			      size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	char *buf = big_buffer;
	ssize_t buf_size;
	ssize_t res;
	unsigned long flags;
	u64 tsf;

	mutex_lock(&big_buffer_mutex);
	buf_size = min(count, ARRAY_SIZE(big_buffer) - 1);
	if (copy_from_user(buf, user_buf, buf_size)) {
		res = -EFAULT;
		goto out_unlock_bb;
	}
	mutex_lock(&dev->wl->mutex);
	spin_lock_irqsave(&dev->wl->irq_lock, flags);
	if (b43legacy_status(dev) < B43legacy_STAT_STARTED) {
		b43legacyerr(dev->wl, "debugfs: Board not initialized.\n");
		res = -EFAULT;
		goto out_unlock;
	}
	if (sscanf(buf, "%llu", (unsigned long long *)(&tsf)) != 1) {
		b43legacyerr(dev->wl, "debugfs: Invalid values for TSF\n");
		res = -EINVAL;
		goto out_unlock;
	}
	b43legacy_tsf_write(dev, tsf);
	mmiowb();
	res = buf_size;

out_unlock:
	spin_unlock_irqrestore(&dev->wl->irq_lock, flags);
	mutex_unlock(&dev->wl->mutex);
out_unlock_bb:
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t power_read_file(struct file *file, char __user *userbuf,
			     size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	const size_t len = ARRAY_SIZE(big_buffer);
	char *buf = big_buffer;
	size_t pos = 0;
	ssize_t res;
	unsigned long flags;

	mutex_lock(&big_buffer_mutex);
	mutex_lock(&dev->wl->mutex);
	spin_lock_irqsave(&dev->wl->irq_lock, flags);
	if (b43legacy_status(dev) < B43legacy_STAT_STARTED) {
		fappend("Board not initialized.\n");
		goto out;
	}
	fappend("%d dBm\n", dev->phy.power_level);

out:
	spin_unlock_irqrestore(&dev->wl->irq_lock, flags);
	mutex_unlock(&dev->wl->mutex);
	res = simple_read_from_buffer(userbuf, count, ppos, buf, pos);
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t power_write_file(struct file *file, const char __user *user_buf,
			      size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	char *buf = big_buffer;
	ssize_t buf_size;
	ssize_t res;
	unsigned long flags;
	int power;

	mutex_lock(&big_buffer_mutex);
	buf_size = min(count, ARRAY_SIZE(big_buffer) - 1);
	if (copy_from_user(buf, user_buf, buf_size)) {
		res = -EFAULT;
		goto out_unlock_bb;
	}
	mutex_lock(&dev->wl->mutex);
	spin_lock_irqsave(&dev->wl->irq_lock, flags);
	if (b43legacy_status(dev) < B43legacy_STAT_STARTED) {
		b43legacyerr(dev->wl, "debugfs: Board not initialized.\n");
		res = -EFAULT;
		goto out_unlock;
	}
	if ((sscanf(buf, "%d", &power) != 1) || (power > 18 || power < 5)) {
		b43legacyerr(dev->wl, "debugfs: Invalid values for power"
			     " level\n");
		res = -EINVAL;
		goto out_unlock;
	}
	dev->phy.power_level = power;
	res = buf_size;

out_unlock:
	spin_unlock_irqrestore(&dev->wl->irq_lock, flags);
	mutex_unlock(&dev->wl->mutex);
out_unlock_bb:
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t txstat_read_file(struct file *file, char __user *userbuf,
				size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	struct b43legacy_dfsentry *e = dev->dfsentry;
	struct b43legacy_txstatus_log *log = &e->txstatlog;
	unsigned long flags;
	char *buf = log->printbuf;
	const size_t len = ARRAY_SIZE(log->printbuf);
	size_t pos = 0;
	ssize_t res;
	int i;
	int idx;
	struct b43legacy_txstatus *stat;

	mutex_lock(&big_buffer_mutex);
	spin_lock_irqsave(&log->lock, flags);
	if (!log->printing) {
		log->printing = 1;
		fappend("b43legacy TX status reports:\n\n"
			"index | cookie | seq | phy_stat | frame_count | "
			"rts_count | supp_reason | pm_indicated | "
			"intermediate | for_ampdu | acked\n"
			"---\n");
		i = log->end + 1;
		idx = 0;
		while (1) {
			if (log->end < 0) {
				fappend("Nothing transmitted, yet\n");
				break;
			}
			if (i == B43legacy_NR_LOGGED_TXSTATUS)
				i = 0;
			stat = &(log->log[i]);
			if (stat->cookie) {
				fappend("%03d | "
					"0x%04X | 0x%04X | 0x%02X | "
					"0x%X | 0x%X | "
					"%u | %u | "
					"%u | %u | %u\n",
					idx,
					stat->cookie, stat->seq, stat->phy_stat,
					stat->frame_count, stat->rts_count,
					stat->supp_reason, stat->pm_indicated,
					stat->intermediate, stat->for_ampdu,
					stat->acked);
				idx++;
			}
			if (i == log->end)
				break;
			i++;
		}
		log->buf_avail = pos;
	}
	memcpy(big_buffer, buf,
	       min(log->buf_avail, ARRAY_SIZE(big_buffer)));
	spin_unlock_irqrestore(&log->lock, flags);

	res = simple_read_from_buffer(userbuf, count, ppos,
				      big_buffer,
				      log->buf_avail);
	if (*ppos == log->buf_avail) {
		spin_lock_irqsave(&log->lock, flags);
		log->printing = 0;
		spin_unlock_irqrestore(&log->lock, flags);
	}
	mutex_unlock(&big_buffer_mutex);

	return res;
}

static ssize_t restart_write_file(struct file *file,
				  const char __user *user_buf,
				  size_t count, loff_t *ppos)
{
	struct b43legacy_wldev *dev = file->private_data;
	char *buf = big_buffer;
	ssize_t buf_size;
	ssize_t res;
	unsigned long flags;

	mutex_lock(&big_buffer_mutex);
	buf_size = min(count, ARRAY_SIZE(big_buffer) - 1);
	if (copy_from_user(buf, user_buf, buf_size)) {
		res = -EFAULT;
		goto out_unlock_bb;
	}
	mutex_lock(&dev->wl->mutex);
	spin_lock_irqsave(&dev->wl->irq_lock, flags);
	if (b43legacy_status(dev) < B43legacy_STAT_INITIALIZED) {
		b43legacyerr(dev->wl, "debugfs: Board not initialized.\n");
		res = -EFAULT;
		goto out_unlock;
	}
	if (count > 0 && buf[0] == '1') {
		b43legacy_controller_restart(dev, "manually restarted");
		res = count;
	} else
		res = -EINVAL;

out_unlock:
	spin_unlock_irqrestore(&dev->wl->irq_lock, flags);
	mutex_unlock(&dev->wl->mutex);
out_unlock_bb:
	mutex_unlock(&big_buffer_mutex);

	return res;
}

#undef fappend


static struct file_operations drvinfo_fops = {
	.read = drvinfo_read_file,
	.write = write_file_dummy,
	.open = open_file_generic,
};

static struct file_operations tsf_fops = {
	.read = tsf_read_file,
	.write = tsf_write_file,
	.open = open_file_generic,
};

static struct file_operations txstat_fops = {
	.read = txstat_read_file,
	.write = write_file_dummy,
	.open = open_file_generic,
};

static struct file_operations restart_fops = {
	.write = restart_write_file,
	.open = open_file_generic,
};

static struct file_operations power_fops = {
	.read = power_read_file,
	.write = power_write_file,
	.open = open_file_generic,
};


int b43legacy_debug(struct b43legacy_wldev *dev,
		    enum b43legacy_dyndbg feature)
{
	return !!(dev->dfsentry && dev->dfsentry->dyn_debug[feature]);
}

static void b43legacy_remove_dynamic_debug(struct b43legacy_wldev *dev)
{
	struct b43legacy_dfsentry *e = dev->dfsentry;
	int i;

	for (i = 0; i < __B43legacy_NR_DYNDBG; i++)
		debugfs_remove(e->dyn_debug_dentries[i]);
}

static void b43legacy_add_dynamic_debug(struct b43legacy_wldev *dev)
{
	struct b43legacy_dfsentry *e = dev->dfsentry;
	struct dentry *d;

#define add_dyn_dbg(name, id, initstate) do {		\
	e->dyn_debug[id] = (initstate);			\
	d = debugfs_create_bool(name, 0600, e->subdir,	\
				&(e->dyn_debug[id]));	\
	if (!IS_ERR(d))					\
		e->dyn_debug_dentries[id] = d;		\
				} while (0)

	add_dyn_dbg("debug_xmitpower", B43legacy_DBG_XMITPOWER, 0);
	add_dyn_dbg("debug_dmaoverflow", B43legacy_DBG_DMAOVERFLOW, 0);
	add_dyn_dbg("debug_dmaverbose", B43legacy_DBG_DMAVERBOSE, 0);
	add_dyn_dbg("debug_pwork_fast", B43legacy_DBG_PWORK_FAST, 0);
	add_dyn_dbg("debug_pwork_stop", B43legacy_DBG_PWORK_STOP, 0);

#undef add_dyn_dbg
}

void b43legacy_debugfs_add_device(struct b43legacy_wldev *dev)
{
	struct b43legacy_dfsentry *e;
	struct b43legacy_txstatus_log *log;
	char devdir[16];

	B43legacy_BUG_ON(!dev);
	e = kzalloc(sizeof(*e), GFP_KERNEL);
	if (!e) {
		b43legacyerr(dev->wl, "debugfs: OOM while adding device\n");
		return;
	}
	e->dev = dev;
	log = &e->txstatlog;
	log->log = kcalloc(B43legacy_NR_LOGGED_TXSTATUS,
			   sizeof(struct b43legacy_txstatus),
			   GFP_KERNEL);
	if (!log->log) {
		b43legacyerr(dev->wl, "debugfs: OOM while adding device"
			     " txstatus\n");
		kfree(e);
		return;
	}
	log->end = -1;
	spin_lock_init(&log->lock);

	dev->dfsentry = e;

	snprintf(devdir, sizeof(devdir), "%s", wiphy_name(dev->wl->hw->wiphy));
	e->subdir = debugfs_create_dir(devdir, fs.root);
	if (!e->subdir || IS_ERR(e->subdir)) {
		b43legacyerr(dev->wl, "debugfs: cannot create %s directory\n",
		       devdir);
		dev->dfsentry = NULL;
		kfree(log->log);
		kfree(e);
		return;
	}

	e->dentry_tsf = debugfs_create_file("tsf", 0600, e->subdir,
					     dev, &tsf_fops);
	if (IS_ERR(e->dentry_tsf))
		e->dentry_tsf = NULL;
	e->dentry_txstat = debugfs_create_file("tx_status", 0400, e->subdir,
						dev, &txstat_fops);
	if (IS_ERR(e->dentry_txstat))
		e->dentry_txstat = NULL;
	e->dentry_restart = debugfs_create_file("restart", 0200, e->subdir,
						dev, &restart_fops);
	if (IS_ERR(e->dentry_restart))
		e->dentry_restart = NULL;

	e->dentry_power = debugfs_create_file("power_level", 0600, e->subdir,
					     dev, &power_fops);
	if (IS_ERR(e->dentry_power))
		e->dentry_power = NULL;

	b43legacy_add_dynamic_debug(dev);
}

void b43legacy_debugfs_remove_device(struct b43legacy_wldev *dev)
{
	struct b43legacy_dfsentry *e;

	if (!dev)
		return;

	e = dev->dfsentry;
	if (!e)
		return;
	b43legacy_remove_dynamic_debug(dev);
	debugfs_remove(e->dentry_tsf);
	debugfs_remove(e->dentry_txstat);
	debugfs_remove(e->dentry_restart);
	debugfs_remove(e->dentry_power);
	debugfs_remove(e->subdir);
	kfree(e->txstatlog.log);
	kfree(e);
}

void b43legacy_debugfs_log_txstat(struct b43legacy_wldev *dev,
				  const struct b43legacy_txstatus *status)
{
	struct b43legacy_dfsentry *e = dev->dfsentry;
	struct b43legacy_txstatus_log *log;
	struct b43legacy_txstatus *cur;
	int i;

	if (!e)
		return;
	log = &e->txstatlog;
	B43legacy_WARN_ON(!irqs_disabled());
	spin_lock(&log->lock);
	i = log->end + 1;
	if (i == B43legacy_NR_LOGGED_TXSTATUS)
		i = 0;
	log->end = i;
	cur = &(log->log[i]);
	memcpy(cur, status, sizeof(*cur));
	spin_unlock(&log->lock);
}

void b43legacy_debugfs_init(void)
{
	memset(&fs, 0, sizeof(fs));
	fs.root = debugfs_create_dir(KBUILD_MODNAME, NULL);
	if (!fs.root || IS_ERR(fs.root)) {
		fs.root = NULL;
		return;
	}
	fs.dentry_driverinfo = debugfs_create_file("driver", 0444, fs.root,
						   NULL, &drvinfo_fops);
	if (IS_ERR(fs.dentry_driverinfo))
		fs.dentry_driverinfo = NULL;
}

void b43legacy_debugfs_exit(void)
{
	debugfs_remove(fs.dentry_driverinfo);
	debugfs_remove(fs.root);
}
